@@ -1,6 +1,6 @@
-;;; uudecode.el -- elisp native uudecode
+;;; uudecode.el -- elisp native uudecode  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1998-2019 Free Software Foundation, Inc.
+;; Copyright (C) 1998-2020 Free Software Foundation, Inc.
 
 ;; Author: Shenghuo Zhu <zsh@cs.rochester.edu>
 ;; Keywords: uudecode news
@@ -24,13 +24,10 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
-
-(eval-and-compile
-  (defalias 'uudecode-char-int
-    (if (fboundp 'char-int)
-	'char-int
-      'identity)))
+(defalias 'uudecode-char-int
+  (if (fboundp 'char-int)
+      'char-int
+    'identity))
 
 (defgroup uudecode nil
   "Decoding of uuencoded data."
@@ -41,19 +38,16 @@
   "Non-nil value should be a string that names a uu decoder.
 The program should expect to read uu data on its standard
 input and write the converted data to its standard output."
-  :type 'string
-  :group 'uudecode)
+  :type 'string)
 
 (defcustom uudecode-decoder-switches nil
   "List of command line flags passed to `uudecode-decoder-program'."
-  :group 'uudecode
   :type '(repeat string))
 
 (defcustom uudecode-use-external
   (executable-find uudecode-decoder-program)
   "Use external uudecode program."
   :version "22.1"
-  :group 'uudecode
   :type 'boolean)
 
 (defconst uudecode-alphabet "\040-\140")
@@ -78,7 +72,7 @@ input and write the converted data to its standard output."
 If FILE-NAME is non-nil, save the result to FILE-NAME.  The program
 used is specified by `uudecode-decoder-program'."
   (interactive "r\nP")
-  (let ((cbuf (current-buffer)) tempfile firstline status)
+  (let ((cbuf (current-buffer)) tempfile firstline)
     (save-excursion
       (goto-char start)
       (when (re-search-forward uudecode-begin-line nil t)
@@ -100,17 +94,13 @@ used is specified by `uudecode-decoder-program'."
 			      (make-temp-name "uu")
 			      uudecode-temporary-file-directory))))
 	(let ((cdir default-directory)
-	      (default-process-coding-system
-		(if (featurep 'xemacs)
-		    ;; In XEmacs, nil is not a valid coding system.
-		    '(binary . binary)
-		  nil)))
+	      (default-process-coding-system nil))
 	  (unwind-protect
 	      (with-temp-buffer
 		(insert "begin 600 " (file-name-nondirectory tempfile) "\n")
 		(insert-buffer-substring cbuf firstline end)
 		(cd (file-name-directory tempfile))
-		(apply 'call-process-region
+		(apply #'call-process-region
 		       (point-min)
 		       (point-max)
 		       uudecode-decoder-program
@@ -128,20 +118,6 @@ used is specified by `uudecode-decoder-program'."
 	  (message "Can not uudecode")))
       (ignore-errors (or file-name (delete-file tempfile))))))
 
-(eval-and-compile
-  (defalias 'uudecode-string-to-multibyte
-    (cond
-     ((featurep 'xemacs)
-      'identity)
-     ((fboundp 'string-to-multibyte)
-      'string-to-multibyte)
-     (t
-      (lambda (string)
-	"Return a multibyte string with the same individual chars as string."
-	(mapconcat
-	 (lambda (ch) (string-as-multibyte (char-to-string ch)))
-	 string ""))))))
-
 ;;;###autoload
 (defun uudecode-decode-region-internal (start end &optional file-name)
   "Uudecode region between START and END without using an external program.
@@ -186,14 +162,12 @@ If FILE-NAME is non-nil, save the result to FILE-NAME."
 	      (setq counter (1+ counter)
 		    inputpos (1+ inputpos))
 	      (cond ((= counter 4)
-		     (setq result (cons
-				   (concat
-				    (char-to-string (lsh bits -16))
-				    (char-to-string (logand (lsh bits -8) 255))
-				    (char-to-string (logand bits 255)))
-				   result))
+		     (setq result (cons (logand bits 255)
+					(cons (logand (ash bits -8) 255)
+					      (cons (ash bits -16)
+						    result))))
 		     (setq bits 0 counter 0))
-		    (t (setq bits (lsh bits 6)))))))
+		    (t (setq bits (ash bits 6)))))))
 	  (cond
 	   (done)
 	   ((> 0 remain)
@@ -203,26 +177,26 @@ If FILE-NAME is non-nil, save the result to FILE-NAME."
 	    ;;(error "uucode ends unexpectedly")
 	    (setq done t))
 	   ((= counter 3)
-	    (setq result (cons
-			  (concat
-			   (char-to-string (logand (lsh bits -16) 255))
-			   (char-to-string (logand (lsh bits -8) 255)))
-			  result)))
+	    (setq result (cons (logand (ash bits -8) 255)
+			       (cons (logand (ash bits -16) 255)
+				     result))))
 	   ((= counter 2)
-	    (setq result (cons
-			  (char-to-string (logand (lsh bits -10) 255))
-			  result))))
+	    (setq result (cons (logand (ash bits -10) 255)
+			       result))))
 	  (skip-chars-forward non-data-chars end))
 	(if file-name
             (with-temp-file file-name
-              (unless (featurep 'xemacs) (set-buffer-multibyte nil))
-              (insert (apply 'concat (nreverse result))))
+              (set-buffer-multibyte nil)
+              (apply #'insert (nreverse result)))
 	  (or (markerp end) (setq end (set-marker (make-marker) end)))
 	  (goto-char start)
-	  (if enable-multibyte-characters
-	      (dolist (x (nreverse result))
-                (insert (uudecode-string-to-multibyte x)))
-	    (insert (apply 'concat (nreverse result))))
+          (apply #'insert
+                 (nreverse
+                  (if enable-multibyte-characters
+                      (mapcar (lambda (ch)
+                                (or (decode-char 'eight-bit ch) ch))
+                              result)
+                    result)))
 	  (delete-region (point) end))))))
 
 ;;;###autoload
