@@ -1,9 +1,8 @@
 ;;; mh-compat.el --- make MH-E compatible with various versions of Emacs
 
-;; Copyright (C) 2006-2017 Free Software Foundation, Inc.
+;; Copyright (C) 2006-2020 Free Software Foundation, Inc.
 
 ;; Author: Bill Wohler <wohler@newt.com>
-;; Maintainer: Bill Wohler <wohler@newt.com>
 ;; Keywords: mail
 ;; See: mh-e.el
 
@@ -65,7 +64,8 @@ Simulate NOERROR argument in XEmacs which lacks it."
 Case is ignored if CASE-FOLD is non-nil.
 This function is used by Emacs versions that lack `assoc-string',
 introduced in Emacs 22."
-  (if case-fold
+  ;; Test for fboundp is solely to silence compiler for Emacs >= 22.1.
+  (if (and case-fold (fboundp 'assoc-ignore-case))
       (assoc-ignore-case key list)
     (assoc key list)))
 
@@ -134,7 +134,7 @@ introduced in Emacs 22."
 
 (defmacro mh-face-background (face &optional frame inherit)
   "Return the background color name of face, or nil if unspecified.
-See documentation for `back-foreground' for a description of the
+See documentation for `face-background' for a description of the
 arguments FACE, FRAME, and INHERIT.
 This macro is used by Emacs versions that lack an INHERIT argument,
 introduced in Emacs 22."
@@ -143,7 +143,7 @@ introduced in Emacs 22."
     `(face-background ,face ,frame ,inherit)))
 
 (defun-mh mh-font-lock-add-keywords font-lock-add-keywords
-  (mode keywords &optional how)
+  (_mode _keywords &optional _how)
   "XEmacs does not have `font-lock-add-keywords'.
 This function returns nil on that system.")
 
@@ -243,7 +243,7 @@ compatibility with versions of Emacs that lack the variable
            (delete image-directory (copy-sequence (or path load-path))))))
 
 (defun-mh mh-image-search-load-path
-  image-search-load-path (file &optional path)
+  image-search-load-path (_file &optional _path)
   "Emacs 21 and XEmacs don't have `image-search-load-path'.
 This function returns nil on those systems."
   nil)
@@ -292,7 +292,7 @@ introduced in Emacs 24."
       `(make-obsolete-variable ,obsolete-name ,current-name ,when ,access-type))))
 
 (defun-mh mh-match-string-no-properties
-  match-string-no-properties (num &optional string)
+  match-string-no-properties (num &optional _string)
   "Return string of text matched by last search, without text properties.
 This function is used by XEmacs that lacks `match-string-no-properties'.
 The function `buffer-substring-no-properties' is used instead.
@@ -301,16 +301,17 @@ The argument STRING is ignored."
    (match-beginning num) (match-end num)))
 
 (defun-mh mh-replace-regexp-in-string replace-regexp-in-string
-  (regexp rep string &optional fixedcase literal subexp start)
+  (regexp rep string &optional _fixedcase literal _subexp _start)
   "Replace REGEXP with REP everywhere in STRING and return result.
 This function is used by XEmacs that lacks `replace-regexp-in-string'.
 The function `replace-in-string' is used instead.
 The arguments FIXEDCASE, SUBEXP, and START, used by
 `replace-in-string' are ignored."
-  (replace-in-string string regexp rep literal))
+  (if (featurep 'xemacs)                ; silence Emacs compiler
+      (replace-in-string string regexp rep literal)))
 
 (defun-mh mh-test-completion
-  test-completion (string collection &optional predicate)
+  test-completion (_string _collection &optional _predicate)
   "Return non-nil if STRING is a valid completion.
 XEmacs does not have `test-completion'. This function returns nil
 on that system." nil)
@@ -351,7 +352,7 @@ The arguments RETURN-TO and EXIT-ACTION are ignored."
   (view-mode 1))
 
 (defun-mh mh-window-full-height-p
-  window-full-height-p (&optional WINDOW)
+  window-full-height-p (&optional _window)
   "Return non-nil if WINDOW is not the result of a vertical split.
 This function is defined in XEmacs as it lacks
 `window-full-height-p'. The values of the functions
