@@ -1,6 +1,6 @@
 ;;; notifications.el --- Client interface to desktop notifications.
 
-;; Copyright (C) 2010-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2010-2020 Free Software Foundation, Inc.
 
 ;; Author: Julien Danjou <julien@danjou.info>
 ;; Keywords: comm desktop notifications
@@ -108,7 +108,7 @@
 	(setq notifications-on-action-object nil)))))
 
 (defun notifications-on-closed-signal (id &optional reason)
-  "Dispatch signals to callback functions from `notifications-on-closed-map'."
+  "Dispatch signals to callback functions from `notifications-on-close-map'."
   ;; notification-daemon prior 0.4.0 does not send a reason.  So we
   ;; make it optional, and assume `undefined' as default.
   (let* ((bus (dbus-event-bus-name last-input-event))
@@ -232,8 +232,8 @@ of another `notifications-notify' call."
 	(add-to-list 'hints `(:dict-entry
 			      "urgency"
 			      (:variant :byte ,(pcase urgency
-						 (`low 0)
-						 (`critical 2)
+						 ('low 0)
+						 ('critical 2)
 						 (_ 1)))) t))
       (when category
 	(add-to-list 'hints `(:dict-entry
