@@ -145,9 +145,17 @@ Possible values are:
 
 (defun diff-no-select (old new &optional switches no-async buf)
   ;; Noninteractive helper for creating and reverting diff buffers
-  "Compare the OLD and NEW file/buffer, and return a diff buffer.
+  "Compare the OLD and NEW file/buffer.
+If the optional SWITCHES is nil, the switches specified in the
+variable ‘diff-switches’ are passed to the diff command,
+otherwise SWITCHES is used.  SWITCHES can be a string or a list
+of strings.
 
-See `diff' for the meaning of the arguments."
+If NO-ASYNC is non-nil, call diff synchronously.
+
+By default, this function creates the diff in the \"*Diff*\"
+buffer.  If BUF is non-nil, BUF is used instead.  This function
+returns the buffer used."
   (unless (bufferp new) (setq new (expand-file-name new)))
   (unless (bufferp old) (setq old (expand-file-name old)))
   (or switches (setq switches diff-switches)) ; If not specified, use default.
