@@ -1624,8 +1624,7 @@ With prefix argument, prompt for cvs flags."
 (defun-cvs-mode (cvs-mode-diff . DOUBLE) (flags)
   "Diff the selected files against the repository.
 This command compares the files in your working area against the
-revision which they are based upon.
-See also `cvs-diff-ignore-marks'."
+revision which they are based upon."
   (interactive
    (list (cvs-add-branch-prefix
 	  (cvs-add-secondary-branch-prefix
