@@ -1,6 +1,6 @@
 ;;; shell-tests.el  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2010-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2010-2020 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -30,4 +30,19 @@
   "Test problem found by Filipp Gunbin in emacs-devel."
   (should (equal (car (shell--unquote&requote-argument "te'st" 2)) "test")))
 
+(ert-deftest shell-tests-completion-before-semi ()
+  (with-temp-buffer
+    (shell-mode)
+    (insert "cd ba;")
+    (forward-char -1)
+    (should (equal (shell--parse-pcomplete-arguments)
+                   '(("cd" "ba") 1 4)))))
+
+(ert-deftest shell-tests-completion-after-semi ()
+  (with-temp-buffer
+    (shell-mode)
+    (insert "cd ba;")
+    (should (equal (shell--parse-pcomplete-arguments)
+                   '(("cd" "ba" "") 1 4 7)))))
+
 ;;; shell-tests.el ends here
